@@ -18,7 +18,6 @@ Contains facilities to open and execute projects without GUI.
 from copy import deepcopy
 from enum import IntEnum, unique
 import json
-import logging
 import pathlib
 import sys
 from PySide2.QtCore import QCoreApplication, QEvent, QObject, QSettings, Signal, Slot
@@ -38,6 +37,7 @@ from .helpers import (
     load_project_dict,
     load_local_project_data,
     merge_dicts,
+    HTMLTagFilter,
 )
 from .spine_engine_manager import make_engine_manager
 
@@ -72,36 +72,37 @@ class HeadlessLogger(QObject):
         self.msg_proc_error.connect(self._log_error)
         self.information_box.connect(self._show_information_box)
         self.error_box.connect(self._show_error_box)
+        self._tag_filter = HTMLTagFilter()
 
-    # pylint: disable=no-self-use
     @Slot(str)
     def _log_message(self, message):
-        """Writes an information message to Python's logging system."""
-        logging.info(message)
+        """Prints an information message."""
+        self._print(message, sys.stdout)
 
-    # pylint: disable=no-self-use
     @Slot(str)
     def _log_warning(self, message):
-        """Writes a warning message to Python's logging system."""
-        logging.warning(message)
+        """Prints a warning message."""
+        self._print(message, sys.stdout)
 
-    # pylint: disable=no-self-use
     @Slot(str)
     def _log_error(self, message):
-        """Writes an error message to Python's logging system."""
-        logging.error(message)
+        """Prints an error message."""
+        self._print(message, sys.stderr)
 
-    # pylint: disable=no-self-use
     @Slot(str, str)
     def _show_information_box(self, title, message):
-        """Writes an information message with a title to Python's logging system."""
-        logging.info(title + ": " + message)
+        """Prints an information message with a title."""
+        self._print(title + ": " + message, sys.stdout)
 
-    # pylint: disable=no-self-use
     @Slot(str, str)
     def _show_error_box(self, title, message):
-        """Writes an error message with a title to Python's logging system."""
-        logging.error(title + ": " + message)
+        """Prints an error message with a title."""
+        self._print(title + ": " + message, sys.stderr)
+
+    def _print(self, message, out_stream):
+        """Filters HTML tags from message before printing it to given file."""
+        self._tag_filter.feed(message)
+        print(self._tag_filter.drain(), file=out_stream)
 
 
 class ModifiableProject:
@@ -297,7 +298,7 @@ class ActionsWithProject(QObject):
             return Status.ERROR
         with open(script_path, encoding="utf-8") as script_file:
             script_code = script_file.read()
-        self._logger.msg.emit(f"Applying {script_path} to project.")
+        self._logger.msg.emit(f"Applying {script_path.name} to project.")
         project = ModifiableProject(self._project_dir, self._item_dicts, self._connection_dicts)
         exec(script_code, {"project": project})
         self._item_dicts = project.items_to_dict()
@@ -370,6 +371,7 @@ class ActionsWithProject(QObject):
             "event_msg": self._handle_event_msg,
             "process_msg": self._handle_process_msg,
             "standard_execution_msg": self._handle_standard_execution_msg,
+            "persistent_execution_msg": self._handle_persistent_execution_msg,
             "kernel_execution_msg": self._handle_kernel_execution_msg,
         }.get(event_type)
         if handler is None:
@@ -392,7 +394,7 @@ class ActionsWithProject(QObject):
         if data["direction"] == "BACKWARD":
             # Currently there are no interesting messages when executing backwards.
             return
-        self._node_messages[data["item_name"]] = list()
+        self._node_messages[data["item_name"]] = dict()
 
     def _handle_node_execution_finished(self, data):
         """Prints messages for finished nodes.
@@ -404,8 +406,11 @@ class ActionsWithProject(QObject):
         messages = self._node_messages.get(item_name)
         if messages is None:
             return
-        for message in messages:
-            self._logger.msg.emit(message)
+        for filter_id, message in messages.items():
+            if filter_id:
+                self._logger.msg.emit(f"--- Output from filter id '{filter_id}' START")
+            for line in message:
+                self._logger.msg.emit(line)
         del self._node_messages[item_name]
 
     def _handle_event_msg(self, data):
@@ -417,7 +422,7 @@ class ActionsWithProject(QObject):
         messages = self._node_messages.get(data["item_name"])
         if messages is None:
             return
-        messages.append(data["msg_text"])
+        messages.setdefault(data["filter_id"], []).append(data["msg_text"])
 
     def _handle_process_msg(self, data):
         """Stores process messages for later printing.
@@ -428,7 +433,7 @@ class ActionsWithProject(QObject):
         messages = self._node_messages.get(data["item_name"])
         if messages is None:
             return
-        messages.append(data["msg_text"])
+        messages.setdefault(data["filter_id"], []).append(data["msg_text"])
 
     def _handle_standard_execution_msg(self, data):
         """Handles standard execution messages.
@@ -439,13 +444,25 @@ class ActionsWithProject(QObject):
             data (dict): execution message data
         """
 
+    def _handle_persistent_execution_msg(self, data):
+        """Handles persistent execution messages.
+
+        Args:
+            data (dict): execution message data
+        """
+        if data["type"] == "stdout" or data["type"] == "stderr":
+            messages = self._node_messages.get(data["item_name"])
+            if messages is None:
+                return
+            messages.setdefault(data["filter_id"], []).append(data["data"])
+
     def _handle_kernel_execution_msg(self, data):
         """Handles kernel messages.
 
         Currently, these messages are ignored.
 
         Args:
-            data (dict): execution message data
+            data (dict): message data
         """
 
 
