@@ -17,8 +17,8 @@ Provides FilterCheckboxListModel for FilterWidget.
 """
 
 import re
-import bisect
 from PySide2.QtCore import Qt, QModelIndex, QAbstractListModel
+from spinetoolbox.helpers import bisect_chunks
 
 
 class SimpleFilterCheckboxListModel(QAbstractListModel):
@@ -42,7 +42,7 @@ class SimpleFilterCheckboxListModel(QAbstractListModel):
         self._base_filter_index = []
         self._is_filtered = False
         self._filter_index = []
-        self._filter_expression = ""
+        self._filter_expression = re.compile("")
         self._all_selected = True
         self._empty_selected = True
         self._add_to_selection = False
@@ -178,7 +178,7 @@ class SimpleFilterCheckboxListModel(QAbstractListModel):
     def set_filter(self, filter_expression):
         if filter_expression and (isinstance(filter_expression, str) and not filter_expression.isspace()):
             self._action_rows[0] = self._SELECT_ALL_STR
-            self._filter_expression = filter_expression
+            self._filter_expression = re.compile(filter_expression)
             self._filter_index = [i for i, item in enumerate(self._data) if self.search_filter_expression(item)]
             self._selected_filtered = set(self._data[i] for i in self._filter_index)
             self._add_to_selection = False
@@ -192,7 +192,7 @@ class SimpleFilterCheckboxListModel(QAbstractListModel):
             self.remove_filter()
 
     def search_filter_expression(self, item):
-        return re.search(self._filter_expression, item)
+        return self._filter_expression.search(item)
 
     def set_base_filter(self, condition):
         """Sets the base filter. The other filter, the one that works by typing in the search bar, should be applied
@@ -240,7 +240,7 @@ class SimpleFilterCheckboxListModel(QAbstractListModel):
             return
         self.beginResetModel()
         self._action_rows[0] = self._SELECT_ALL_STR
-        self._filter_expression = ""
+        self._filter_expression = re.compile("")
         if self._show_add_to_selection:
             self._action_rows.remove(self._ADD_TO_SELECTION_STR)
         self._filter_index = self._base_filter_index
@@ -290,8 +290,7 @@ class SimpleFilterCheckboxListModel(QAbstractListModel):
 
 
 class LazyFilterCheckboxListModel(SimpleFilterCheckboxListModel):
-    """Extends SimpleFilterCheckboxListModel to allow for lazy loading in synch with another model.
-    """
+    """Extends SimpleFilterCheckboxListModel to allow for lazy loading in synch with another model."""
 
     def __init__(self, parent, source_model, show_empty=True):
         """Init class.
@@ -303,48 +302,30 @@ class LazyFilterCheckboxListModel(SimpleFilterCheckboxListModel):
         super().__init__(parent, show_empty=show_empty)
         self.source_model = source_model
 
-    def canFetchMore(self, parent=QModelIndex()):
+    def canFetchMore(self, parent):
         if self.source_model is None:
             return False
-        return self.source_model.canFetchMore()
+        return self.source_model.canFetchMore(QModelIndex())
 
-    def fetchMore(self, parent=QModelIndex()):
+    def fetchMore(self, parent):
         if self.source_model is None:
             return
-        row_count_before = self.rowCount()
-        self.source_model.fetchMore()
+        row_count_before = self.rowCount(parent)
+        self.source_model.fetchMore(QModelIndex())
         # If fetching the source model doesn't bring any new data, emit layoutChanged to fetch more again.
-        if self.rowCount() == row_count_before:
+        if self.rowCount(parent) == row_count_before:
             self.layoutChanged.emit()
 
     def _do_add_items(self, data):
-        """Adds items so the list is always sorted, while assuming that both existing and new items are sorted.
-        """
-        data_iter = iter(data)
-        item = next(data_iter)
-        consecutive_items = [item]
-        lo = bisect.bisect_left(self._data, item)
-        for item in data_iter:
-            row = bisect.bisect_left(self._data, item, lo=lo)
-            if row == lo:
-                consecutive_items.append(item)
-                continue
-            count = self._do_insert_rows(consecutive_items, lo)
-            consecutive_items = [item]
-            lo = row + count
-        self._do_insert_rows(consecutive_items, lo)
-
-    def _do_insert_rows(self, rows, pos):
-        count = len(rows)
-        self.beginInsertRows(self.index(0, 0), pos, pos + count - 1)
-        self._data[pos:pos] = rows
-        self.endInsertRows()
-        return count
+        """Adds items so the list is always sorted, while assuming that both existing and new items are sorted."""
+        for chunk, pos in bisect_chunks(self._data, data):
+            self.beginInsertRows(self.index(0, 0), pos, pos + len(chunk) - 1)
+            self._data[pos:pos] = chunk
+            self.endInsertRows()
 
 
 class DataToValueFilterCheckboxListModel(SimpleFilterCheckboxListModel):
-    """Extends SimpleFilterCheckboxListModel to allow for translating internal data to a value for display role.
-    """
+    """Extends SimpleFilterCheckboxListModel to allow for translating internal data to a value for display role."""
 
     def __init__(self, parent, data_to_value, show_empty=True):
         """Init class.
@@ -363,4 +344,4 @@ class DataToValueFilterCheckboxListModel(SimpleFilterCheckboxListModel):
         return data
 
     def search_filter_expression(self, item):
-        return re.search(self._filter_expression, self.data_to_value(item))
+        return self._filter_expression.search(self.data_to_value(item))
