@@ -20,7 +20,7 @@ The main entrance points to plotting are:
 - add_time_series_plot() which adds a time series plot to an existing PlotWidget
 - add_map_plot() which adds a map plot to an existing PlotWidget
 
-:author: A. Soininen(VTT)
+:author: A. Soininen (VTT)
 :date:   9.7.2019
 """
 
@@ -28,7 +28,7 @@ import functools
 from numbers import Number
 from matplotlib.ticker import MaxNLocator
 import numpy as np
-from PySide2.QtCore import QModelIndex
+from PySide2.QtCore import Qt, QModelIndex
 from spinedb_api import (
     Array,
     convert_leaf_maps_to_specialized_containers,
@@ -42,6 +42,9 @@ from .mvcmodels.shared import PARSED_ROLE
 from .widgets.plot_widget import PlotWidget
 
 
+_PLOT_SETTINGS = {"alpha": 0.7}
+
+
 class PlottingError(Exception):
     """An exception signalling failure in plotting."""
 
@@ -83,9 +86,9 @@ def plot_pivot_column(proxy_model, column, hints, plot_widget=None):
             plot_widget.infer_plot_type(values)
         else:
             _raise_if_value_types_clash(values, plot_widget)
-    _add_plot_to_widget(values, labels, plot_widget)
+    add_plot_to_widget(values, labels, plot_widget)
     if len(plot_widget.canvas.axes.get_lines()) > 1:
-        plot_widget.canvas.axes.legend(loc="best", fontsize="small")
+        plot_widget.add_legend()
     plot_widget.canvas.axes.set_xlabel(hints.x_label(proxy_model))
     plot_lines = plot_widget.canvas.axes.get_lines()
     if plot_lines:
@@ -113,21 +116,21 @@ def plot_selection(model, indexes, hints, plot_widget=None):
     else:
         needs_redraw = True
     selections = hints.filter_columns(_organize_selection_to_columns(indexes), model)
-    all_labels = list()
     for column, rows in selections.items():
         values, labels = _collect_column_values(model, column, rows, hints)
-        all_labels += labels
+        plot_widget.all_labels += labels
         if values:
             if plot_widget.plot_type is None:
                 plot_widget.infer_plot_type(values)
             else:
                 _raise_if_value_types_clash(values, plot_widget)
-        _add_plot_to_widget(values, labels, plot_widget)
+        add_plot_to_widget(values, labels, plot_widget)
     plot_widget.canvas.axes.set_xlabel(hints.x_label(model))
-    if len(all_labels) > 1:
-        plot_widget.canvas.axes.legend(loc="best", fontsize="small")
-    elif len(all_labels) == 1:
-        plot_widget.canvas.axes.set_title(all_labels[0])
+    if len(plot_widget.all_labels) > 1:
+        plot_widget.canvas.axes.set_title("")
+        plot_widget.add_legend()
+    elif len(plot_widget.all_labels) == 1:
+        plot_widget.canvas.axes.set_title(plot_widget.all_labels[0])
     if needs_redraw:
         plot_widget.canvas.draw()
     return plot_widget
@@ -142,7 +145,7 @@ def add_array_plot(plot_widget, value, label=None):
         value (Array): the array to plot
         label (str): a label for the array
     """
-    plot_widget.canvas.axes.plot(value.indexes, value.values, label=label)
+    plot_widget.canvas.axes.plot(value.indexes, value.values, label=label, **_PLOT_SETTINGS)
 
 
 def add_map_plot(plot_widget, map_value, label=None):
@@ -164,7 +167,9 @@ def add_map_plot(plot_widget, map_value, label=None):
         indexes_as_strings = list(map(str, map_value.indexes))
     else:
         indexes_as_strings = map_value.indexes
-    plot_widget.canvas.axes.plot(indexes_as_strings, map_value.values, label=label, linestyle="", marker="o")
+    plot_widget.canvas.axes.plot(
+        indexes_as_strings, map_value.values, label=label, linestyle="", marker="o", **_PLOT_SETTINGS
+    )
     plot_widget.canvas.axes.xaxis.set_major_locator(MaxNLocator(10))
 
 
@@ -177,13 +182,14 @@ def add_time_series_plot(plot_widget, value, label=None):
         value (TimeSeries): the time series to plot
         label (str): a label for the time series
     """
-    plot_widget.canvas.axes.step(value.indexes, value.values, label=label, where='post')
+    plot_widget.canvas.axes.step(value.indexes, value.values, label=label, where='post', **_PLOT_SETTINGS)
     # matplotlib cannot have time stamps before 0001-01-01T00:00 on the x axis
     left, _ = plot_widget.canvas.axes.get_xlim()
     if left < 1.0:
         # 1.0 corresponds to 0001-01-01T00:00
         plot_widget.canvas.axes.set_xlim(left=1.0)
-    plot_widget.canvas.figure.autofmt_xdate()
+    # FIXME: The below causes xticklabels to disappear when plotting legend as a subplot
+    # plot_widget.canvas.figure.autofmt_xdate()
 
 
 class PlottingHints:
@@ -223,6 +229,34 @@ class PlottingHints:
         raise NotImplementedError()
 
 
+class MapTablePlottingHints(PlottingHints):
+    """Support for plotting data in Parameter table views."""
+
+    def cell_label(self, model, index):
+        """Returns a label build from the columns on the left from the data column."""
+        return model.index_name(index)
+
+    def column_label(self, model, column):
+        """Returns the column header."""
+        return model.headerData(column, orientation=Qt.Horizontal)
+
+    def filter_columns(self, selections, model):
+        """Returns the selections unaltered."""
+        return selections
+
+    def is_index_in_data(self, model, index):
+        """Always returns True."""
+        return True
+
+    def special_x_values(self, model, column, rows):
+        """Always returns None."""
+        return None
+
+    def x_label(self, model):
+        """Returns an empty string for the x axis label."""
+        return ""
+
+
 class ParameterTablePlottingHints(PlottingHints):
     """Support for plotting data in Parameter table views."""
 
@@ -325,7 +359,7 @@ class PivotTablePlottingHints(PlottingHints):
         return proxy_model.mapFromSource(source_index).column()
 
 
-def _add_plot_to_widget(values, labels, plot_widget):
+def add_plot_to_widget(values, labels, plot_widget):
     """Adds a new plot to plot_widget."""
     if not values:
         return
@@ -339,7 +373,7 @@ def _add_plot_to_widget(values, labels, plot_widget):
         for value, label in zip(values, labels):
             add_array_plot(plot_widget, value, label)
     elif isinstance(values[1][0], Number):
-        plot_widget.canvas.axes.plot(values[0], values[1], label=labels[0])
+        plot_widget.canvas.axes.plot(values[0], values[1], label=labels[0], **_PLOT_SETTINGS)
         if isinstance(values[0][0], str):
             # matplotlib tries to plot every single x tick label if they are strings.
             # This can become very slow if the labels are numerous.
@@ -351,7 +385,7 @@ def _add_plot_to_widget(values, labels, plot_widget):
 def _raise_if_not_all_indexed_values(values):
     """Raises an exception if not all values are TimeSeries or Maps."""
     if not values:
-        return values
+        return
     first_value_type = type(values[0])
     if issubclass(first_value_type, TimeSeries):
         # Clump fixed and variable step time series together. We can plot both at the same time.
@@ -373,9 +407,7 @@ def _organize_selection_to_columns(indexes):
     selections = dict()
     for index in indexes:
         selections.setdefault(index.column(), set()).add(index.row())
-    for column, rows in selections.items():
-        selections[column] = list(sorted(rows))
-    return selections
+    return {column: sorted(rows) for column, rows in selections.items()}
 
 
 def _collect_single_column_values(model, column, rows, hints):
@@ -494,8 +526,8 @@ def _collect_column_values(model, column, rows, hints):
     values, labels = _collect_single_column_values(model, column, rows, hints)
     if not values:
         return values, labels
-    if isinstance(first_non_null(values), Map):
-        values, labels = _expand_maps(values, labels)
+    if len(values) == len(labels):
+        values, labels = expand_maps(values, labels)
     if isinstance(first_non_null(values), (Array, Map, TimeSeries)):
         values = [x for x in values if x is not None]
         _raise_if_not_all_indexed_values(values)
@@ -511,7 +543,7 @@ def _collect_column_values(model, column, rows, hints):
     return (usable_x, usable_y), labels
 
 
-def _expand_maps(maps, labels):
+def expand_maps(maps, labels):
     """
     Gathers the leaf elements from ``maps`` and expands ``labels`` accordingly.
 
@@ -527,6 +559,10 @@ def _expand_maps(maps, labels):
     for map_, label in zip(maps, labels):
         if map_ is None:
             continue
+        if not isinstance(map_, Map):
+            expanded_values.append(map_)
+            expanded_labels.append(label)
+            continue
         map_ = convert_leaf_maps_to_specialized_containers(map_)
         if isinstance(map_, (Array, TimeSeries)):
             expanded_values.append(map_)
@@ -540,7 +576,7 @@ def _expand_maps(maps, labels):
 
 def _label_nested_maps(map_, label):
     """
-    Collects leaf values from given Maps and labels them.
+    Collects leaf values from given Map and labels them.
 
     Args:
         map_ (Map): a map
