@@ -16,7 +16,7 @@ Models to represent things in a tree.
 """
 from PySide2.QtCore import Qt, QModelIndex
 from spinetoolbox.mvcmodels.minimal_tree_model import MinimalTreeModel
-from .tree_item_utility import NonLazyTreeItem
+from .tree_item_utility import StandardTreeItem
 
 
 class TreeModelBase(MinimalTreeModel):
@@ -49,7 +49,7 @@ class TreeModelBase(MinimalTreeModel):
     def build_tree(self):
         """Builds tree."""
         self.beginResetModel()
-        self._invisible_root_item = NonLazyTreeItem(self)
+        self._invisible_root_item = StandardTreeItem(self)
         self.endResetModel()
         for db_map in self.db_maps:
             db_item = self._make_db_item(db_map)
@@ -65,22 +65,20 @@ class TreeModelBase(MinimalTreeModel):
         raise NotImplementedError()
 
     def _items_per_db_item(self, db_map_data):
-        d = {}
         for db_item in self._invisible_root_item.children:
             items = db_map_data.get(db_item.db_map)
             if not items:
                 continue
-            d[db_item] = items
-        return d
+            yield db_item, items
 
-    def _ids_per_root_item(self, db_map_data, root_number=0):
+    def _items_per_root(self, db_map_data, root_number=0):
         d = {}
         for db_item in self._invisible_root_item.children:
             items = db_map_data.get(db_item.db_map)
             if not items:
                 continue
             root_item = db_item.child(root_number)
-            d[root_item] = [x["id"] for x in items]
+            d[root_item] = items
         return d
 
     @staticmethod
@@ -107,12 +105,34 @@ class TreeModelBase(MinimalTreeModel):
     @staticmethod
     def _remove_leaf_items(root_item, ids):
         removed_rows = []
-        for row, leaf_item in enumerate(root_item.children[:-1]):
+        for row, leaf_item in enumerate(root_item.children):
             if leaf_item.id and leaf_item.id in ids:
                 removed_rows.append(row)
         for row in sorted(removed_rows, reverse=True):
             root_item.remove_children(row, 1)
 
+    @staticmethod
+    def _insert_items(parent_item, db_items, make_child):
+        """Inserts items at right positions. Items with commit_id are kept sorted.
+        Items without a commit_id are put at the end.
+
+        Args:
+            parent_item (TreeItem)
+            db_items (list of dict): database items
+            make_child (function): A function that receives an integer id and returns a TreeItem
+        """
+        ids_committed = []
+        ids_uncommitted = []
+        for item in db_items:
+            if item["id"] in parent_item.children_ids:
+                continue
+            ids = ids_committed if item.get("commit_id") is not None else ids_uncommitted
+            ids.append(item["id"])
+        children_committed = [make_child(id_) for id_ in ids_committed]
+        children_uncommitted = [make_child(id_) for id_ in ids_uncommitted]
+        parent_item.insert_children_sorted(children_committed)
+        parent_item.insert_children(len(parent_item.non_empty_children), children_uncommitted)
+
     @staticmethod
     def db_item(item):
         while item.item_type != "db":
