@@ -17,9 +17,9 @@ Contains the MultiSpineDBEditor class.
 """
 
 import os
-from PySide2.QtWidgets import QToolBar, QWidget, QSizePolicy, QMenu
-from PySide2.QtCore import Qt, Slot, QPoint, QSize
-from PySide2.QtGui import QIcon
+from PySide2.QtWidgets import QMessageBox, QMenu, QStatusBar, QToolButton
+from PySide2.QtCore import Slot, QPoint
+from PySide2.QtGui import QIcon, QFont
 from .spine_db_editor import SpineDBEditor
 from .custom_qwidgets import ShootingLabel, OpenFileButton, OpenSQLiteFileButton
 from ...widgets.multi_tab_window import MultiTabWindow
@@ -30,16 +30,25 @@ from ...config import MAINWINDOW_SS
 
 
 class MultiSpineDBEditor(MultiTabWindow):
+    """Database editor's tabbed main window."""
+
     def __init__(self, db_mngr, db_url_codenames=None):
+        """
+        Args:
+            db_mngr (SpineDBManager): database manager
+            db_url_codenames (dict, optional): mapping from database URL to its codename
+        """
         super().__init__(db_mngr.qsettings, "spineDBEditor")
         self.db_mngr = db_mngr
+        self.db_mngr.waiting_for_fetcher.connect(self._show_waiting_for_fetcher)
+        self.db_mngr.fetcher_waiting_over.connect(self._close_waiting_for_fetcher)
+        self._waiting_box = None
         self.settings_form = SpineDBEditorSettingsWidget(self)
         self.setStyleSheet(MAINWINDOW_SS)
         self.setWindowTitle("Spine DB Editor")
         self.setWindowIcon(QIcon(":/symbols/app.ico"))
-        self._file_open_toolbar = _FileOpenToolBar(self)
-        self._file_open_toolbar.hide()
-        self.addToolBar(Qt.BottomToolBarArea, self._file_open_toolbar)
+        self.setStatusBar(_CustomStatusBar(self))
+        self.statusBar().hide()
         if db_url_codenames is not None:
             self.add_new_tab(db_url_codenames)
 
@@ -111,14 +120,13 @@ class MultiSpineDBEditor(MultiTabWindow):
             button (OpenFileButton)
         """
         duplicates = [
-            x
-            for x in self._file_open_toolbar.findChildren(OpenFileButton)
-            if os.path.samefile(x.file_path, button.file_path)
+            x for x in self.statusBar().findChildren(OpenFileButton) if os.path.samefile(x.file_path, button.file_path)
         ]
         for dup in duplicates:
-            self._file_open_toolbar.remove_widget(dup)
-        self._file_open_toolbar.preppend_widget(button)
-        self._file_open_toolbar.show()
+            self.statusBar().removeWidget(dup)
+            dup.deleteLater()
+        self.statusBar().insertWidget(0, button)
+        self.statusBar().show()
         destination = QPoint(16, 0) + button.mapTo(self, QPoint(0, 0))
         label = ShootingLabel(destination - QPoint(0, 64), destination, self)
         pixmap = QIcon(":/icons/file-download.svg").pixmap(32, 32)
@@ -146,26 +154,49 @@ class MultiSpineDBEditor(MultiTabWindow):
         if not open_url(doc_url):
             self.msg_error.emit("Unable to open url <b>{0}</b>".format(doc_url))
 
+    @Slot()
+    def _show_waiting_for_fetcher(self):
+        """Shows a message box to user informing that a tab is waiting for fetcher to finish working."""
+        if self._waiting_box is not None:
+            return
+        self._waiting_box = QMessageBox(
+            QMessageBox.Information, "Closing database", "Waiting for database to close...", QMessageBox.Ok, parent=self
+        )
+        self._waiting_box.button(QMessageBox.Ok).setText("I wait patiently")
+        self._waiting_box.open()
 
-class _FileOpenToolBar(QToolBar):
+    @Slot()
+    def _close_waiting_for_fetcher(self):
+        if self._waiting_box is None:
+            return
+        self._waiting_box.close()
+        self._waiting_box.deleteLater()
+        self._waiting_box = None
+
+
+class _CustomStatusBar(QStatusBar):
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.setObjectName("multi_spine_db_editor_file_open_toolbar")
-        empty = QWidget()
-        empty.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
-        self.addWidget(empty)
-        self.addAction(QIcon(CharIconEngine("\uf00d")), "Close", self.hide)
-        self.setMovable(False)
-        self.setIconSize(QSize(20, 20))
-
-    def preppend_widget(self, widget):
-        first_action = next(iter(self.actions()), None)
-        if first_action:
-            self.insertWidget(first_action, widget)
-        else:
-            self.addWidget(widget)
-
-    def remove_widget(self, widget):
-        action = next(iter(a for a in self.actions() if self.widgetForAction(a) is widget), None)
-        if action:
-            self.removeAction(action)
+        self.setContentsMargins(0, 0, 4, 0)
+        self._hide_button = QToolButton()
+        self._hide_button.setStyleSheet(
+            """
+            QToolButton {
+                background-color: transparent;
+                border: 0px;
+                border-radius: 6px;
+            }
+            QToolButton:hover {
+                background-color: #dddddd;
+            }
+            QToolButton:pressed {
+                background-color: #bbbbbb;
+            }
+            """
+        )
+        self._hide_button.setText("\uf00d")
+        self._hide_button.setFont(QFont('Font Awesome 5 Free Solid'))
+        self._hide_button.setFixedSize(24, 24)
+        self.insertPermanentWidget(0, self._hide_button)
+        self.setSizeGripEnabled(False)
+        self._hide_button.clicked.connect(self.hide)
