@@ -15,7 +15,7 @@ from pygments.styles import get_style_by_name
 from pygments.lexers import get_lexer_by_name
 from pygments.util import ClassNotFound
 from pygments.token import Token
-from PySide2.QtCore import Qt, Slot, QTimer, Signal, QThreadPool, QRunnable, QMutex, QMutexLocker
+from PySide2.QtCore import Qt, Slot, QTimer, Signal
 from PySide2.QtWidgets import QTextEdit
 from PySide2.QtGui import QFontDatabase, QTextCharFormat, QFont, QTextCursor, QColor, QTextBlockFormat, QTextOption
 from spinetoolbox.helpers import CustomSyntaxHighlighter
@@ -27,9 +27,11 @@ class PersistentConsoleWidget(QTextEdit):
 
     _history_item_available = Signal(str, str)
     _completions_available = Signal(str, str, list)
+    _flush_needed = Signal()
     _FLUSH_INTERVAL = 200
     _MAX_LINES_PER_SECOND = 2000
     _MAX_LINES_PER_CYCLE = _MAX_LINES_PER_SECOND * 1000 / _FLUSH_INTERVAL
+    _MAX_LINES_COUNT = 2000
 
     def __init__(self, toolbox, key, language, owner=None):
         """
@@ -45,12 +47,9 @@ class PersistentConsoleWidget(QTextEdit):
         cursor_width = self.fontMetrics().horizontalAdvance("x")
         self.setCursorWidth(cursor_width)
         self.document().setIndentWidth(cursor_width)
-        self.document().setMaximumBlockCount(2000)
+        self.document().setMaximumBlockCount(self._MAX_LINES_COUNT)
         self.setWordWrapMode(QTextOption.WrapAnywhere)
         self.setTabStopDistance(4 * cursor_width)
-        self._pool = QThreadPool()
-        self._text_buffer_pool = QThreadPool()
-        self._text_buffer_pool.setMaxThreadCount(1)
         self._toolbox = toolbox
         self._key = key
         self._language = language
@@ -60,11 +59,8 @@ class PersistentConsoleWidget(QTextEdit):
         self._reset_prefix = True
         self._pending_command_count = 0
         self._text_buffer = []
-        self._text_buffer_mutex = QMutex()
         self._skipped = {}
         self._anchor = None
-        self._flush_timer = QTimer()
-        self._flush_timer.setInterval(self._FLUSH_INTERVAL)
         self._style = get_style_by_name("monokai")
         background_color = self._style.background_color
         foreground_color = self._style.styles[Token] or self._style.styles[Token.Text]
@@ -86,8 +82,12 @@ class PersistentConsoleWidget(QTextEdit):
         self.document().contentsChanged.connect(self._handle_contents_changed)
         self._history_item_available.connect(self._display_history_item)
         self._completions_available.connect(self._display_completions)
+        self._flush_needed.connect(self._start_flush_timer)
+        self._flush_in_progess = False
+        self._flush_timer = QTimer()
+        self._flush_timer.setInterval(self._FLUSH_INTERVAL)
         self._flush_timer.timeout.connect(self._flush_text_buffer)
-        self._flush_timer.start()
+        self._flush_timer.setSingleShot(True)
 
     def name(self):
         """Returns console name for display purposes."""
@@ -98,7 +98,7 @@ class PersistentConsoleWidget(QTextEdit):
         return " & ".join(x.name for x in self.owners if x is not None)
 
     def mouseMoveEvent(self, ev):
-        super().mousePressEvent(ev)
+        super().mouseMoveEvent(ev)
         if self.anchorAt(ev.pos()):
             self.viewport().setCursor(Qt.PointingHandCursor)
         else:
@@ -130,6 +130,7 @@ class PersistentConsoleWidget(QTextEdit):
         scrollbar = self.verticalScrollBar()
         self._at_bottom = scrollbar.value() == scrollbar.maximum()
 
+    @Slot()
     def _handle_contents_changed(self):
         if self._at_bottom:
             scrollbar = self.verticalScrollBar()
@@ -215,23 +216,21 @@ class PersistentConsoleWidget(QTextEdit):
         Args:
             text (str)
         """
-        self._text_buffer_pool.start(_CustomRunnable(self._do_insert_text_before_prompt, text, with_prompt))
-
-    def _do_insert_text_before_prompt(self, text, with_prompt=False):
-        QMutexLocker(self._text_buffer_mutex)
         self._text_buffer.append((text, with_prompt))
+        if not self._flush_in_progess:
+            self._flush_in_progess = True
+            self._flush_needed.emit()
+
+    @Slot()
+    def _start_flush_timer(self):
+        self._flush_timer.start()
 
     @Slot()
     def _flush_text_buffer(self):
         """Inserts all text from buffer."""
-        self._text_buffer_pool.start(_CustomRunnable(self._do_flush_text_buffer))
-
-    def _do_flush_text_buffer(self):
-        QMutexLocker(self._text_buffer_mutex)
-        self.blockSignals(True)
-        k = 0
         cursor = self.textCursor()
         cursor.beginEditBlock()
+        k = 0
         while self._text_buffer and k < self._MAX_LINES_PER_CYCLE:
             cursor.setPosition(self._prompt_block.position() - 1)
             text, with_prompt = self._text_buffer.pop(0)
@@ -244,13 +243,13 @@ class PersistentConsoleWidget(QTextEdit):
             char_format.setForeground(QColor('blue'))
             char_format.setAnchor(True)
             char_format.setAnchorHref(address)
-            self._skipped[address] = self._text_buffer.copy()
+            self._skipped[address] = self._text_buffer[-self._MAX_LINES_COUNT :]
             cursor.setPosition(self._prompt_block.position() - 1)
             cursor.insertBlock(QTextBlockFormat())
             cursor.insertText(f"<--- {len(self._text_buffer)} more lines --->", char_format)
             self._text_buffer.clear()
         cursor.endEditBlock()
-        self.blockSignals(False)
+        self._flush_in_progess = False
 
     def _insert_text(self, cursor, text, with_prompt):
         cursor.insertBlock(QTextBlockFormat())
@@ -349,7 +348,7 @@ class PersistentConsoleWidget(QTextEdit):
         log_stdin = bool(self._pending_command_count)
         self._make_prompt_block(prompt="")
         self._pending_command_count += 1
-        self._pool.start(_CustomRunnable(self._do_issue_command, engine_mngr, text, log_stdin))
+        self._do_issue_command(engine_mngr, text, log_stdin)
 
     def _do_issue_command(self, engine_mngr, text, log_stdin):
         for msg in engine_mngr.issue_persistent_command(self._key, text):
@@ -369,9 +368,6 @@ class PersistentConsoleWidget(QTextEdit):
 
     def _move_history(self, text, backwards):
         """Moves history."""
-        self._pool.start(_CustomRunnable(self._do_move_history, text, backwards))
-
-    def _do_move_history(self, text, backwards):
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
         prefix = self._get_prefix()
@@ -398,9 +394,6 @@ class PersistentConsoleWidget(QTextEdit):
         Args:
             text (str)
         """
-        self._pool.start(_CustomRunnable(self._do_autocomplete, text))
-
-    def _do_autocomplete(self, text):
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
         prefix = self._get_prefix()
@@ -427,9 +420,7 @@ class PersistentConsoleWidget(QTextEdit):
         self.clear()
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        self._pool.start(_CustomRunnable(self._do_restart_persistent, engine_mngr))
-
-    def _do_restart_persistent(self, engine_mngr):
+        self._text_buffer.clear()
         for msg in engine_mngr.restart_persistent(self._key):
             msg_type = msg["type"]
             if msg_type == "stdout":
@@ -443,7 +434,7 @@ class PersistentConsoleWidget(QTextEdit):
         """Interrupts underlying persistent process."""
         engine_server_address = self._toolbox.qsettings().value("appSettings/engineServerAddress", defaultValue="")
         engine_mngr = make_engine_manager(engine_server_address)
-        self._pool.start(_CustomRunnable(engine_mngr.interrupt_persistent, self._key))
+        engine_mngr.interrupt_persistent(self._key)
 
     def _extend_menu(self, menu):
         """Adds two more actions: Restart, and Interrupt."""
@@ -458,17 +449,6 @@ class PersistentConsoleWidget(QTextEdit):
         menu.exec_(event.globalPos())
 
 
-class _CustomRunnable(QRunnable):
-    def __init__(self, function_to_run, *args, **kwargs):
-        super().__init__(self)
-        self._function_to_run = function_to_run
-        self._args = args
-        self._kwargs = kwargs
-
-    def run(self):
-        self._function_to_run(*self._args, **self._kwargs)
-
-
 # Translated from
 # https://code.qt.io/cgit/qt-creator/qt-creator.git/tree/src/libs/utils/ansiescapecodehandler.cpp?h=master
 # TODO: Consider qtconsole's QtAnsiCodeProcessor
