@@ -31,7 +31,8 @@ from .notification import Notification
 from .install_julia_wizard import InstallJuliaWizard
 from .add_up_spine_opt_wizard import AddUpSpineOptWizard
 from ..config import DEFAULT_WORK_DIR, SETTINGS_SS
-from ..link import Link
+from ..link import Link, JumpLink
+from ..project_item_icon import ProjectItemIcon
 from ..widgets.kernel_editor import (
     KernelEditor,
     MiniPythonKernelEditor,
@@ -156,6 +157,7 @@ class SpineDBEditorSettingsMixin:
         """Connect signals."""
         super().connect_signals()
         self.ui.checkBox_auto_expand_objects.clicked.connect(self.set_auto_expand_objects)
+        self.ui.checkBox_merge_dbs.clicked.connect(self.set_merge_dbs)
 
     def read_settings(self):
         """Read saved settings from app QSettings instance and update UI to display them."""
@@ -165,6 +167,7 @@ class SpineDBEditorSettingsMixin:
         smooth_rotation = self._qsettings.value("appSettings/smoothEntityGraphRotation", defaultValue="false")
         relationship_items_follow = self._qsettings.value("appSettings/relationshipItemsFollow", defaultValue="true")
         auto_expand_objects = self._qsettings.value("appSettings/autoExpandObjects", defaultValue="true")
+        merge_dbs = self._qsettings.value("appSettings/mergeDBs", defaultValue="true")
         db_editor_show_undo = int(self._qsettings.value("appSettings/dbEditorShowUndo", defaultValue="2"))
         if commit_at_exit == 0:  # Not needed but makes the code more readable.
             self.ui.checkBox_commit_at_exit.setCheckState(Qt.Unchecked)
@@ -177,6 +180,7 @@ class SpineDBEditorSettingsMixin:
         self.ui.checkBox_smooth_entity_graph_rotation.setChecked(smooth_rotation == "true")
         self.ui.checkBox_relationship_items_follow.setChecked(relationship_items_follow == "true")
         self.ui.checkBox_auto_expand_objects.setChecked(auto_expand_objects == "true")
+        self.ui.checkBox_merge_dbs.setChecked(merge_dbs == "true")
         if db_editor_show_undo == 2:
             self.ui.checkBox_db_editor_show_undo.setChecked(True)
 
@@ -196,6 +200,8 @@ class SpineDBEditorSettingsMixin:
         self._qsettings.setValue("appSettings/relationshipItemsFollow", relationship_items_follow)
         auto_expand_objects = "true" if int(self.ui.checkBox_auto_expand_objects.checkState()) else "false"
         self._qsettings.setValue("appSettings/autoExpandObjects", auto_expand_objects)
+        merge_dbs = "true" if int(self.ui.checkBox_merge_dbs.checkState()) else "false"
+        self._qsettings.setValue("appSettings/mergeDBs", merge_dbs)
         db_editor_show_undo = str(int(self.ui.checkBox_db_editor_show_undo.checkState()))
         self._qsettings.setValue("appSettings/dbEditorShowUndo", db_editor_show_undo)
         return True
@@ -203,12 +209,23 @@ class SpineDBEditorSettingsMixin:
     def update_ui(self):
         super().update_ui()
         auto_expand_objects = self._qsettings.value("appSettings/autoExpandObjects", defaultValue="true") == "true"
+        merge_dbs = self._qsettings.value("appSettings/mergeDBs", defaultValue="true") == "true"
         self.set_auto_expand_objects(auto_expand_objects)
+        self.set_merge_dbs(merge_dbs)
+
+    @Slot(bool)
+    def set_auto_expand_objects(self, checked=False):
+        for db_editor in self.db_mngr.get_all_spine_db_editors():
+            db_editor.ui.graphicsView.set_auto_expand_objects(checked)
+
+    @Slot(bool)
+    def set_merge_dbs(self, checked=False):
+        for db_editor in self.db_mngr.get_all_spine_db_editors():
+            db_editor.ui.graphicsView.set_merge_dbs(checked)
 
 
 class SpineDBEditorSettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
-    """A widget to change user's preferred settings, but only for the Spine db editor.
-    """
+    """A widget to change user's preferred settings, but only for the Spine db editor."""
 
     def __init__(self, multi_db_editor):
         """Initialize class."""
@@ -222,10 +239,9 @@ class SpineDBEditorSettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase
         super().show()
         self.read_settings()
 
-    @Slot(bool)
-    def set_auto_expand_objects(self, checked=False):
-        for db_editor in self._multi_db_editor.db_mngr.get_all_spine_db_editors():
-            db_editor.ui.graphicsView.set_auto_expand_objects(checked)
+    @property
+    def db_mngr(self):
+        return self._multi_db_editor.db_mngr
 
 
 class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
@@ -252,11 +268,8 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         self.setAttribute(Qt.WA_DeleteOnClose)
         self.connect_signals()
         self.read_settings()
-        self.read_project_settings()
-        self._update_python_widgets_enabled(
-            True if self.ui.radioButton_use_python_jupyter_console.isChecked() else False
-        )
-        self._update_julia_widgets_enabled(True if self.ui.radioButton_use_julia_jupyter_console.isChecked() else False)
+        self._update_python_widgets_enabled(self.ui.radioButton_use_python_jupyter_console.isChecked())
+        self._update_julia_widgets_enabled(self.ui.radioButton_use_julia_jupyter_console.isChecked())
 
     def connect_signals(self):
         """Connect signals."""
@@ -275,10 +288,17 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         self.ui.radioButton_bg_solid.clicked.connect(self.update_scene_bg)
         self.ui.checkBox_color_toolbar_icons.clicked.connect(self.set_toolbar_colored_icons)
         self.ui.checkBox_use_curved_links.clicked.connect(self.update_links_geometry)
+        self.ui.checkBox_use_rounded_items.clicked.connect(self.update_items_path)
         self.ui.pushButton_install_julia.clicked.connect(self._show_install_julia_wizard)
         self.ui.pushButton_add_up_spine_opt.clicked.connect(self._show_add_up_spine_opt_wizard)
         self.ui.radioButton_use_python_jupyter_console.toggled.connect(self._update_python_widgets_enabled)
         self.ui.radioButton_use_julia_jupyter_console.toggled.connect(self._update_julia_widgets_enabled)
+        self.ui.user_defined_engine_process_limit_radio_button.toggled.connect(
+            self.ui.engine_process_limit_spin_box.setEnabled
+        )
+        self.ui.user_defined_persistent_process_limit_radio_button.toggled.connect(
+            self.ui.persistent_process_limit_spin_box.setEnabled
+        )
 
     @Slot(bool)
     def _update_python_widgets_enabled(self, state):
@@ -318,10 +338,9 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         wizard = AddUpSpineOptWizard(self, julia_exe, julia_project)
         wizard.show()
 
-    @Slot(bool)
-    def set_auto_expand_objects(self, checked=False):
-        for db_editor in self._toolbox.db_mngr.get_all_spine_db_editors():
-            db_editor.ui.graphicsView.set_auto_expand_objects(checked)
+    @property
+    def db_mngr(self):
+        return self._toolbox.db_mngr
 
     @Slot(bool)
     def browse_gams_button_clicked(self, checked=False):
@@ -467,13 +486,23 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
     @Slot(bool)
     def update_links_geometry(self, checked=False):
         for item in self._toolbox.ui.graphicsView.items():
-            if isinstance(item, Link):
+            if isinstance(item, (Link, JumpLink)):
                 item.update_geometry(curved_links=checked)
 
+    @Slot(bool)
+    def update_items_path(self, checked=False):
+        for item in self._toolbox.ui.graphicsView.items():
+            if isinstance(item, ProjectItemIcon):
+                item.update_path(checked)
+
     @Slot(bool)
     def set_toolbar_colored_icons(self, checked=False):
         self._toolbox.main_toolbar.set_colored_icons(checked)
 
+    @Slot(bool)
+    def _update_properties_widget(self, _checked=False):
+        self._toolbox.ui.tabWidget_item_properties.update()
+
     def read_settings(self):
         """Read saved settings from app QSettings instance and update UI to display them."""
         # checkBox check state 0: unchecked, 1: partially checked, 2: checked
@@ -487,7 +516,10 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         custom_open_project_dialog = self._qsettings.value("appSettings/customOpenProjectDialog", defaultValue="true")
         smooth_zoom = self._qsettings.value("appSettings/smoothZoom", defaultValue="false")
         color_toolbar_icons = self._qsettings.value("appSettings/colorToolbarIcons", defaultValue="false")
+        color_properties_widgets = self._qsettings.value("appSettings/colorPropertiesWidgets", defaultValue="false")
         curved_links = self._qsettings.value("appSettings/curvedLinks", defaultValue="false")
+        rounded_items = self._qsettings.value("appSettings/roundedItems", defaultValue="false")
+        prevent_overlapping = self._qsettings.value("appSettings/preventOverlapping", defaultValue="false")
         data_flow_anim_dur = int(self._qsettings.value("appSettings/dataFlowAnimationDuration", defaultValue="100"))
         bg_choice = self._qsettings.value("appSettings/bgChoice", defaultValue="solid")
         bg_color = self._qsettings.value("appSettings/bgColor", defaultValue="false")
@@ -523,8 +555,15 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
             self.ui.checkBox_use_smooth_zoom.setCheckState(Qt.Checked)
         if color_toolbar_icons == "true":
             self.ui.checkBox_color_toolbar_icons.setCheckState(Qt.Checked)
+        if color_properties_widgets == "true":
+            self.ui.checkBox_color_properties_widgets.setCheckState(Qt.Checked)
         if curved_links == "true":
             self.ui.checkBox_use_curved_links.setCheckState(Qt.Checked)
+        if rounded_items == "true":
+            self.ui.checkBox_use_rounded_items.setCheckState(Qt.Checked)
+        self.ui.horizontalSlider_data_flow_animation_duration.setValue(data_flow_anim_dur)
+        if prevent_overlapping == "true":
+            self.ui.checkBox_prevent_overlapping.setCheckState(Qt.Checked)
         self.ui.horizontalSlider_data_flow_animation_duration.setValue(data_flow_anim_dur)
         if bg_choice == "grid":
             self.ui.radioButton_bg_grid.setChecked(True)
@@ -581,16 +620,30 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
             self.ui.checkBox_save_spec_before_closing.setCheckState(Qt.Checked)
         if spec_show_undo == 2:
             self.ui.checkBox_spec_show_undo.setChecked(True)
+        self._read_engine_settings()
 
-    def read_project_settings(self):
-        """Get project name and description and update widgets accordingly."""
-        if self._project:
-            self.ui.lineEdit_project_name.setText(self._project.name)
-            self.ui.textEdit_project_description.setText(self._project.description)
+    def _read_engine_settings(self):
+        """Reads Engine settings and sets the corresponding UI elements."""
+        parallel_process_limit_choice = self._qsettings.value("engineSettings/processLimiter", defaultValue="auto")
+        if parallel_process_limit_choice == "auto":
+            self.ui.automatic_engine_process_limit_radio_button.setChecked(True)
         else:
-            # Disable project name and description line edits if no project open
-            self.ui.lineEdit_project_name.setDisabled(True)
-            self.ui.textEdit_project_description.setDisabled(True)
+            self.ui.user_defined_engine_process_limit_radio_button.setChecked(True)
+        parallel_process_limit = int(self._qsettings.value("engineSettings/maxProcesses", defaultValue=os.cpu_count()))
+        self.ui.engine_process_limit_spin_box.setValue(parallel_process_limit)
+        persistent_process_limit_choice = self._qsettings.value(
+            "engineSettings/persistentLimiter", defaultValue="unlimited"
+        )
+        if persistent_process_limit_choice == "unlimited":
+            self.ui.unlimited_persistent_process_radio_button.setChecked(True)
+        elif persistent_process_limit_choice == "auto":
+            self.ui.automatic_persistent_process_limit_radio_button.setChecked(True)
+        else:
+            self.ui.user_defined_persistent_process_limit_radio_button.setChecked(True)
+        persistent_process_limit = int(
+            self._qsettings.value("engineSettings/maxPersistentProcesses", defaultValue=os.cpu_count())
+        )
+        self.ui.persistent_process_limit_spin_box.setValue(persistent_process_limit)
 
     @Slot()
     def save_settings(self):
@@ -619,8 +672,14 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         self._qsettings.setValue("appSettings/smoothZoom", smooth_zoom)
         color_toolbar_icons = "true" if int(self.ui.checkBox_color_toolbar_icons.checkState()) else "false"
         self._qsettings.setValue("appSettings/colorToolbarIcons", color_toolbar_icons)
+        color_properties_widgets = "true" if int(self.ui.checkBox_color_properties_widgets.checkState()) else "false"
+        self._qsettings.setValue("appSettings/colorPropertiesWidgets", color_properties_widgets)
         curved_links = "true" if int(self.ui.checkBox_use_curved_links.checkState()) else "false"
         self._qsettings.setValue("appSettings/curvedLinks", curved_links)
+        rounded_items = "true" if int(self.ui.checkBox_use_rounded_items.checkState()) else "false"
+        self._qsettings.setValue("appSettings/roundedItems", rounded_items)
+        prevent_overlapping = "true" if int(self.ui.checkBox_prevent_overlapping.checkState()) else "false"
+        self._qsettings.setValue("appSettings/preventOverlapping", prevent_overlapping)
         data_flow_anim_dur = str(self.ui.horizontalSlider_data_flow_animation_duration.value())
         self._qsettings.setValue("appSettings/dataFlowAnimationDuration", data_flow_anim_dur)
         if self.ui.radioButton_bg_grid.isChecked():
@@ -684,8 +743,32 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
         self.set_work_directory(work_dir)
         # Check if something in the app needs to be updated
         self._toolbox.show_datetime = self._toolbox.update_datetime()
-        # Project
-        self.update_project_settings()
+        if not self._save_engine_settings():
+            return False
+        return True
+
+    def _save_engine_settings(self):
+        """Stores Engine settings to application settings.
+
+        Returns:
+            bool: True if settings were stored successfully, False otherwise
+        """
+        if self.ui.automatic_engine_process_limit_radio_button.isChecked():
+            limiter = "auto"
+        else:
+            limiter = "user"
+        self._qsettings.setValue("engineSettings/processLimiter", limiter)
+        self._qsettings.setValue("engineSettings/maxProcesses", str(self.ui.engine_process_limit_spin_box.value()))
+        if self.ui.unlimited_persistent_process_radio_button.isChecked():
+            limiter = "unlimited"
+        elif self.ui.automatic_persistent_process_limit_radio_button.isChecked():
+            limiter = "auto"
+        else:
+            limiter = "user"
+        self._qsettings.setValue("engineSettings/persistentLimiter", limiter)
+        self._qsettings.setValue(
+            "engineSettings/maxPersistentProcesses", str(self.ui.persistent_process_limit_spin_box.value())
+        )
         return True
 
     def _get_julia_settings(self):
@@ -698,19 +781,6 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
             julia_kernel = self.ui.comboBox_julia_kernel.currentText()
         return use_julia_jupyter_console, julia_exe, julia_project, julia_kernel
 
-    def update_project_settings(self):
-        """Update project name and description if these have been changed."""
-        if not self._project:
-            return
-        new_name = self.ui.lineEdit_project_name.text().strip()
-        if self._project.name != new_name:
-            # Change project name
-            if new_name != "":
-                self._project.call_set_name(new_name)
-        if not self._project.description == self.ui.textEdit_project_description.toPlainText():
-            # Set new project description
-            self._project.call_set_description(self.ui.textEdit_project_description.toPlainText())
-
     def set_work_directory(self, new_work_dir):
         """Sets new work directory.
 
@@ -725,11 +795,13 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
     def update_ui(self):
         super().update_ui()
         curved_links = self._qsettings.value("appSettings/curvedLinks", defaultValue="false")
+        rounded_items = self._qsettings.value("appSettings/roundedItems", defaultValue="false")
         bg_choice = self._qsettings.value("appSettings/bgChoice", defaultValue="solid")
         bg_color = self._qsettings.value("appSettings/bgColor", defaultValue="false")
         color_toolbar_icons = self._qsettings.value("appSettings/colorToolbarIcons", defaultValue="false")
         self.set_toolbar_colored_icons(color_toolbar_icons == "true")
         self.update_links_geometry(curved_links == "true")
+        self.update_items_path(rounded_items == "true")
         if bg_choice == "grid":
             self.ui.radioButton_bg_grid.setChecked(True)
         elif bg_choice == "tree":
@@ -743,6 +815,10 @@ class SettingsWidget(SpineDBEditorSettingsMixin, SettingsWidgetBase):
             self.bg_color = bg_color
         self.update_bg_color()
 
+    def closeEvent(self, ev):
+        super().closeEvent(ev)
+        self._toolbox.update_properties_ui()
+
 
 def _get_python_kernel_name_by_exe(python_exe):
     """Returns a kernel name corresponding to given python exe, or an empty string if none available.
